#include   <stdio.h>
#include   <stdlib.h>
#include   <visa.h>
#include   "age5270.h"
 
void check_err (ViSession vi, ViStatus ret) {
 ViInt32    	inst_err;
 ViChar		    err_msg[256];
 if(VI_SUCCESS > ret) {
     if ( age5270_INSTR_ERROR_DETECTED == ret ) {
         age5270_error_query(vi, &inst_err, err_msg);
         printf("Instrument Error: %ld\n %s\n", inst_err, err_msg);
	 }
     else {
         age5270_error_message(vi, ret, err_msg);
         printf("Driver Error: %ld\n %s\n", ret, err_msg);
	 }
 }
}

void perform_meas (ViSession vi, ViStatus ret) /* 1 */
{
ViInt32 emitter = 1; /* SMU1 */ /* 3 */
ViInt32 base = 2; /* SMU2 */
ViInt32 collector = 4; /* SMU4 */
ViReal64 vb1 = 0.3;
ViReal64 vb2 = 0.8;
ViReal64 vc = 3;
ViReal64 ve = 0;
ViReal64 ibcomp = 0.01;
ViReal64 iccomp = 0.1;
ViReal64 iecomp = 0.1;
ViReal64 pcomp = 0;
ViInt32 nop = 11;
ViReal64 hold = 0;
ViReal64 delay = 0;
ViReal64 s_delay = 0;
ViReal64 p_comp = 0;
ViReal64 width = 0.001;
ViReal64 period = 0.01;
ViReal64 p_hold = 0.1;
ViInt32 smpl = 5;
ViInt32 mch[3];
ViInt32 mode[2];
ViReal64 range[2];
ViInt32 rep;
ViReal64 sc[11];
ViReal64 md[22];
ViInt32 st[22];
ViReal64 tm[22];
mch[0] = collector;
mch[1] = base;
mch[2] = 0;
mode[0] = 1;
mode[1] = 1;
range[0] = 0;
range[1] = 0; /* 37 */
ret = age5270_setSwitch(vi, emitter, 1); /* 39 */
ret = age5270_setSwitch(vi, base, 1);
ret = age5270_setSwitch(vi, collector, 1);
check_err (vi, ret); /* 42 */
ret = age5270_setAdc(vi, age5270_HSPEED_ADC, age5270_INTEG_MANUAL, smpl,
age5270_FLAG_OFF);
ret = age5270_setAdcType(vi, age5270_CH_ALL, age5270_HSPEED_ADC); /* 45 */
ret = age5270_resetTimestamp(vi);
check_err (vi, ret); /* 48 */
ret = age5270_force(vi, emitter, age5270_VF_MODE, 0, ve, iecomp, 0); /* 50 */
ret = age5270_force(vi, collector, age5270_VF_MODE, 0, vc, iccomp, 0);
ret = age5270_setIv(vi, base, age5270_SWP_VF_SGLLIN, 0, vb1, vb2, nop, hold, delay, s_delay, ibcomp, pcomp);
check_err (vi, ret); /* 53 */
ret = age5270_sweepMiv(vi, mch, mode, range, &rep, &sc[0], &md[0], &st[0], &tm[0]);
check_err (vi, ret); /* 56 */
ret = age5270_zeroOutput(vi, age5270_CH_ALL); /* 58 */
ret = age5270_setSwitch(vi, age5270_CH_ALL, 0);
check_err (vi, ret); /* 60 */
if ( rep != nop ) { /* 62 */
printf ("%d measurement steps were returned.\nIt must be %d steps.\n", rep, nop);
exit (ret);
}
ViInt32 i; /* 67 */
ViInt32 n;
printf(" Vb (V), Ic (mA), Ib (mA)\n");
for (i = 0; i < nop; i++){
printf(" %3.2f, %10.8f, %10.8f\n", sc[i], md[2*i] * 1000, md[2*i+1] * 1000);
}
ViChar f_name[] = "data1.txt"; /* 74 */
ViChar head1[] = "Vb (V), Ic (mA), Ib (mA), hfe, Tc (sec), Tb (sec), Status_c, Status_b";
ViChar msg1[] = "Saving data...";
ViChar msg2[] = "Data save completed.";
ViChar c = '\n';
FILE *stream;
if( ( stream = fopen( f_name, "w+" )) == NULL ){
printf( "Data file was not opened\n" );
}
else {
printf( "%s%c", msg1, c );
fprintf( stream, "%s%c", head1, c );
for (i = 0; i < nop; i++){
fprintf( stream, "%3.2f, %10.8f, %10.8f, %10.8f, %8.6f, %8.6f, %d, %d\n",
sc[i], md[2*i] * 1000, md[2*i+1] * 1000, md[2*i]/md[2*i+1], tm[2*i], tm[2*i+1], st[2*i], st[2*i+1]);
}
printf( "%s%c", msg2, c );
}
if( fclose( stream ) ){
printf( "Data file was not closed\n" );
}
} /* 96 */
 
ViStatus main ( )
{
 ViStatus    ret;
 ViSession   vi;
 ViChar      err_msg[256];

 /* Starting the session. */
 ret = age5270_init("GPIB::17::INSTR", VI_TRUE, VI_TRUE, &vi);
 if ( ( ret < VI_SUCCESS ) || ( vi == VI_NULL ) ) {
     printf("Initialization failure.\n Status code: %d.\n", ret);
     if ( vi != VI_NULL ) {
         age5270_error_message(vi, ret, err_msg);
         printf("Error: %ld\n %s\n", ret, err_msg);
	 }
     exit (ret);
 }
 ret = age5270_reset(vi);              	       /* Resets E5270              */
 ret = age5270_timeOut(vi, 60000); 	           /* sets 60 second timeout    */
 ret = age5270_errorQueryDetect(vi, VI_TRUE);  /* Turns on error detection. */
 
 perform_meas (vi, ret);
 ret = age5270_cmd(vi, "aa");
 check_err(vi, ret);
 
 /* Closing the session. */
 ret = age5270_close(vi);
 check_err(vi, ret);

 return VI_SUCCESS;
}