Imports Agilent.TMFramework
Imports Agilent.TMFramework.DataAnalysis
Imports Agilent.TMFramework.DataVisualization
Imports Agilent.TMFramework.InstrumentIO
Imports Agilent.TMFramework.InstrumentDriverInterop
Imports Agilent.TMFramework.InstrumentDriverInterop.Design
Imports Agilent.TMFramework.InstrumentDriverInterop.VxipnpWrappers

Module Module1

    Sub Main()
        Dim Age527x As Age5270 = New Age5270("GPIB0::23::INSTR", True, True)
        Age527x.Reset()
        Age527x.TimeOut(60000)

        MsgBox("Click OK to start measurement.", vbOKOnly, "")
        Console.WriteLine("Measurement in progress. . ." & Chr(10))
        perform_meas(Age527x)

        Age527x.SetSwitch(0, 0)
        Age527x.Close()
        MsgBox("Click OK to stop the program.", vbOKOnly, "")
        Console.WriteLine("Measurement completed." & Chr(10))
    End Sub

    Sub perform_meas(ByVal Age527x As Age5270)
        Dim i As Integer = 0
        Dim j As Integer = 0
        Dim nop1 As Integer = 1
        Dim nop2 As Integer = 1
        Dim data(nop2, nop1) As String
        Dim val As String = "Id (mA), Time (sec), Status"
        Dim fname As String = "C:\Agilent\data\ex1.txt"
        Dim title As String = "Spot Measurement Result"
        Dim msg As String = "No error."
        Dim err As Integer = 0

        Dim t() As Integer = {2, 5, 6, 7}
        Age527x.SetSwitch(t(3), 1)              'SMU4: substrate
        Age527x.SetSwitch(t(2), 1)              'SMU3: source
        Age527x.SetSwitch(t(1), 1)              'SMU2: gate
        Age527x.SetSwitch(t(0), 1)              'SMU1: drain

        Dim vd As Double = 0.5
        Dim vg As Double = 0.5
        Dim idcomp As Double = 0.05
        Dim igcomp As Double = 0.01
        Dim meas As Double
        Dim status As Integer
        Dim time As Double
        Age527x.Force(t(3), 2, 0, 0, 0.1, 0)    'out= 0 V, comp= 0.1 A
        Age527x.Force(t(2), 2, 0, 0, 0.1, 0)    'out= 0 V, comp= 0.1 A
        Age527x.Force(t(1), 2, 2, vg, igcomp, 0)
        Age527x.Force(t(0), 2, 2, vd, idcomp, 0)
        Age527x.ErrorQuery(err, msg)
        If err <> 0 Then Age527x.ZeroOutput(0) : GoTo Check_err

        Age527x.ResetTimestamp()
        Age527x.SpotMeas(t(0), 1, 0, meas, status, time)
        data(j, i) = Chr(13) & Chr(10) & meas * 1000 & ", " & time & ", " & status

        Age527x.ZeroOutput(0)
        save_data(fname, title, val, data, nop1, nop2, Age527x)

Check_err:
        If err <> 0 Then MsgBox("Instrument error: " & err & Chr(10) & msg, vbOKOnly, "")
    End Sub

    Sub save_data(ByVal fname As String, ByVal title As String, ByVal val As String, ByVal data(,) As String, ByVal nop1 As Integer, ByVal nop2 As Integer, ByVal Age527x As Age5270)
        Dim i As Integer = 0
        Dim j As Integer = 0
        FileOpen(1, fname, OpenMode.Output, OpenAccess.Write, OpenShare.LockReadWrite)
        Print(1, val)
        For j = 0 To nop2
            For i = 0 To nop1
                Print(1, data(j, i))
            Next
        Next
        FileClose(1)

        Dim rbx As Integer
        For j = 0 To nop2
            For i = 0 To nop1
                val = val & data(j, i)
            Next
        Next
        val = val & Chr(10) & Chr(10) & "Data save completed."
        val = val & Chr(10) & Chr(10) & "Do you want to perform measurement again?"
        rbx = MsgBox(val, vbYesNo, title)
        If rbx = vbYes Then perform_meas(Age527x)
    End Sub

End Module
