Imports Agilent.TMFramework
Imports Agilent.TMFramework.DataAnalysis
Imports Agilent.TMFramework.DataVisualization
Imports Agilent.TMFramework.InstrumentIO
Imports Agilent.TMFramework.InstrumentDriverInterop
Imports Agilent.TMFramework.InstrumentDriverInterop.Design
Imports Agilent.TMFramework.InstrumentDriverInterop.VxipnpWrappers

Module Module1

    Sub Main()
        Dim Age527x As Age5270 = New Age5270("GPIB0::23::INSTR", True, True)
        Age527x.Reset()
        Age527x.TimeOut(60000)

        MsgBox("Click OK to start measurement.", vbOKOnly, "")
        Console.WriteLine("Measurement in progress. . ." & Chr(10))
        perform_meas(Age527x)

        Age527x.SetSwitch(0, 0)
        Age527x.Close()
        MsgBox("Click OK to stop the program.", vbOKOnly, "")
        Console.WriteLine("Measurement completed." & Chr(10))
    End Sub

    Sub perform_meas(ByVal Age527x As Age5270)
        Dim i As Integer = 0
        Dim j As Integer = 0
        Dim nop1 As Integer = 1
        Dim nop2 As Integer = 1
        Dim data(nop2, nop1) As String
        Dim val As String = "Ic (mA), Time_c (sec), Status_c, Ib (mA), Time_b (sec), Status_b, hfe"
        Dim fname As String = "C:\Agilent\data\ex2.txt"
        Dim title As String = "Spot Measurement Result"
        Dim msg As String = "No error."
        Dim err As Integer = 0

        Dim t() As Integer = {2, 5, 6}
        Age527x.SetSwitch(t(2), 1)        'SMU6: collector
        Age527x.SetSwitch(t(1), 1)        'SMU5: base
        Age527x.SetSwitch(t(0), 1)        'SMU2: emitter

        Dim vc As Double = 3
        Dim vb As Double = 0.7
        Dim ve As Double = 0
        Dim iccomp As Double = 0.1
        Dim ibcomp As Double = 0.01
        Dim iecomp As Double = 0.1
        Dim mch() As Integer = {t(2), t(1), 0}
        Dim mode() As Integer = {1, 1}   'current measurement
        Dim range() As Double = {0, 0}   'auto range
        Dim md(2) As Double
        Dim st(2) As Integer
        Dim tm(2) As Double

        Age527x.Force(t(0), Age5270.ModeEnum1.VoltageOutput, 0, ve, iecomp, 0)
        Age527x.Force(t(1), Age5270.ModeEnum1.VoltageOutput, 0, vb, ibcomp, 0)
        Age527x.Force(t(2), Age5270.ModeEnum1.VoltageOutput, 0, vc, iccomp, 0)
        Age527x.ErrorQuery(err, msg)
        If err <> 0 Then Age527x.ZeroOutput(0) : GoTo Check_err

        Age527x.ResetTimestamp()
        Age527x.MeasureM(mch, mode, range, md, st, tm)
        data(j, i) = Chr(13) & Chr(10) & md(0) * 1000 & ", " & tm(0) & ", " & st(0)
        data(j, i) = data(j, i) & ", " & md(1) * 1000 & ", " & tm(1) & ", " & st(1)
        data(j, i) = data(j, i) & ", " & md(0) / md(1)

        Age527x.ZeroOutput(0)
        save_data(fname, title, val, data, nop1, nop2, Age527x)

Check_err:
        If err <> 0 Then MsgBox("Instrument error: " & err & Chr(10) & msg, vbOKOnly, "")
    End Sub

    Sub save_data(ByVal fname As String, ByVal title As String, ByVal val As String, ByVal data(,) As String, ByVal nop1 As Integer, ByVal nop2 As Integer, ByVal Age527x As Age5270)
        Dim i As Integer = 0
        Dim j As Integer = 0
        FileOpen(1, fname, OpenMode.Output, OpenAccess.Write, OpenShare.LockReadWrite)
        Print(1, val)
        For j = 0 To nop2 - 1
            For i = 0 To nop1 - 1
                Print(1, data(j, i))
            Next
        Next
        FileClose(1)

        Dim rbx As Integer
        For j = 0 To nop2 - 1
            For i = 0 To nop1 - 1
                val = val & data(j, i)
            Next
        Next
        val = val & Chr(10) & Chr(10) & "Data save completed."
        val = val & Chr(10) & Chr(10) & "Do you want to perform measurement again?"
        rbx = MsgBox(val, vbYesNo, title)
        If rbx = vbYes Then perform_meas(Age527x)
    End Sub

End Module
