Imports Ivi.visa.interop

Module Module1


    Sub Main()
        Dim B1500 As IResourceManager
        Dim session As IMessage
        B1500 = New ResourceManager
        session = B1500.Open("GPIB0::17::INSTR")
        session.WriteString("*RST" & vbLf)
        MsgBox("Click OK to start measurement.", vbOKOnly, "")
        Console.WriteLine("Measurement in progress. . ." & Chr(10))

        Dim t() As Integer = {4, 3, 2, 1}
        Dim term As String = t(0) & "," & t(1) & "," & t(2) & "," & t(3)
        session.WriteString("CN " & term & vbLf)
        perform_meas(session, t)

        session.WriteString("CL" & vbLf)
        session.Close()
        MsgBox("Click OK to stop the program.", vbOKOnly, "")
        Console.WriteLine("Measurement completed." & Chr(10))
    End Sub

    Sub perform_meas(ByVal session As IMessage, ByVal t() As Integer)
        Dim i As Integer = 0                         't(0): Drain
        Dim j As Integer = 0                         't(1): Gate
        Dim nop1 As Integer = 11                     't(2): Source
        Dim nop2 As Integer = 3                      't(3): Substrate
        Dim data(nop2 - 1, nop1 - 1) As String
        Dim value As String = "Vg (V), Vd (V), Id (mA), Time (sec), Status"
        Dim fname As String = "C:\Agilent\prog_ex\data4r.txt"
        Dim title As String = "Measurement Result"
        Dim msg As String = "No error."
        Dim err As Integer = 0

        Dim vd1 As Double = 0
        Dim vd2 As Double = 3
        Dim idcomp As Double = 0.05
        Dim vg1 As Double = 1
        Dim vg2 As Double = 3
        Dim igcomp As Double = 0.01
        Dim vg As Double = vg1                  'secondary sweep output value
        Dim d_vg As Double = 0                  'secondary sweep step value (delta)
        If nop2 <> 1 Then d_vg = (vg2 - vg1) / (nop2 - 1)

        Dim hold As Double = 0
        Dim delay As Double = 0
        Dim s_delay As Double = 0
        Dim p_comp As Double = 0.3

        session.WriteString("FMT 5,1" & vbLf)   'ASCII, <comma>, w/sweep source data
        session.WriteString("TSC 1" & vbLf)     'enables time stamp output
        session.WriteString("FL 0" & vbLf)      'sets filter off
        session.WriteString("AV 10,1" & vbLf)   'sets number of samples for 1 data
        session.WriteString("DV " & t(3) & ",0,0,0.1" & vbLf)   'out= 0 V, comp= 0.1 A
        session.WriteString("DV " & t(2) & ",0,0,0.1" & vbLf)   'out= 0 V, comp= 0.1 A
        session.WriteString("MM 2," & t(0) & vbLf)              '2: staircase sweep measurement
        session.WriteString("CMM " & t(0) & ",1" & vbLf)        '1: current measurement
        session.WriteString("RI " & t(0) & ",0" & vbLf)         '0: auto ranging
        session.WriteString("WT " & hold & "," & delay & "," & s_delay & vbLf)
        session.WriteString("WM 2,1" & vbLf)                    'stops any abnormal
        session.WriteString("ERR? 1" & vbLf) : err = session.ReadString(4 + 2)
        If err <> 0 Then session.WriteString("DZ" & vbLf) : GoTo Check_err

        Dim ret_val As String : Dim status As String : Dim chan As String
        Dim type As String : Dim rdata As Double : Dim tdata As Double
        Dim sdata As Double : Dim mdata As Double : Dim mstat As String
        Dim disp_data As String
        Dim k As Integer = 0
        session.TerminationCharacter = 44       'terminator = comma
        session.TerminationCharacterEnabled = True

        For j = 0 To nop2 - 1
            session.WriteString("WV " & t(0) & ",1,0," & vd1 & "," & vd2 & "," & nop1 & "," & idcomp & "," & p_comp & vbLf)
            session.WriteString("DV " & t(1) & ",0" & "," & vg & "," & igcomp & vbLf)
            session.WriteString("TSR" & vbLf)
            session.WriteString("XE" & vbLf)
            For i = 0 To nop1 - 1
                For k = 0 To 2
                    ret_val = session.ReadString(16)
                    status = Left(ret_val, 1)
                    chan = Mid(ret_val, 2, 1)
                    type = Mid(ret_val, 3, 1)
                    rdata = Val(Mid(ret_val, 4, 12))
                    If type = "T" Then tdata = rdata
                    If type = "I" Then mdata = rdata : mstat = status
                    If type = "V" Then sdata = rdata
                Next k
                If mstat <> "N" Then session.WriteString("DZ" & vbLf) : GoTo Check_err
                disp_data = "Vg = " & vg & " (V), "
                disp_data = disp_data & "Vd = " & sdata & " (V), "
                disp_data = disp_data & "Id = " & mdata * 1000 & " (mA), "
                disp_data = disp_data & "Time = " & tdata & " (sec), "
                disp_data = disp_data & "Status = " & mstat
                Console.WriteLine(disp_data)
                data(j, i) = Chr(13) & Chr(10) & vg & ", " & sdata & ", " & mdata * 1000 & ", " & tdata & ", " & mstat
            Next i
            vg = vg + d_vg
        Next j
        session.WriteString("DZ" & vbLf)
        save_data(fname, title, value, data, nop1, nop2, session, t)
        Exit Sub

Check_err:
        session.WriteString("EMG? " & err & vbLf) : msg = session.ReadString(256)
        MsgBox("Instrument error: " & err & Chr(10) & msg, vbOKOnly, "")
        Exit Sub
    End Sub

    Sub save_data(ByVal fname As String, ByVal title As String, ByVal value As String, ByVal data(,) As String, ByVal nop1 As Integer, ByVal nop2 As Integer, ByVal session As IMessage, ByVal t() As Integer)
        Dim i As Integer = 0
        Dim j As Integer = 0
        FileOpen(1, fname, OpenMode.Output, OpenAccess.Write, OpenShare.LockReadWrite)
        Print(1, value)
        For j = 0 To nop2 - 1
            For i = 0 To nop1 - 1
                Print(1, data(j, i))
            Next
        Next
        FileClose(1)

        Dim rbx As Integer
        For j = 0 To nop2 - 1
            For i = 0 To nop1 - 1
                value = value & data(j, i)
            Next
        Next
        value = value & Chr(10) & Chr(10) & "Data save completed."
        value = value & Chr(10) & Chr(10) & "Do you want to perform measurement again?"
        rbx = MsgBox(value, vbYesNo, title)
        If rbx = vbYes Then perform_meas(session, t)
    End Sub

End Module
